"""
 Copyright (c) 2021-2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 DHT support courtesy of Martyn Wheeler
 Based on the DHTNew library - https://github.com/RobTillaart/DHTNew
"""

import asyncio
import sys

from telemetrix_rpi_pico_2w_serial_aio import telemetrix_rpi_pico_2w_serial_aio


async def dummy_callback(data):
    pass


async def get_pin_report(the_board):
    # set some pins to different modes
    await the_board.set_pin_mode_digital_output(4)
    await the_board.set_pin_mode_digital_input(6, callback=dummy_callback)
    await the_board.set_pin_mode_analog_input(1, callback=dummy_callback)
    await the_board.set_pin_mode_digital_input_pullup(9, callback=dummy_callback)
    await the_board.set_pin_mode_neopixel(14)
    await the_board.set_pin_mode_i2c(0, 4, 5)

    print(await the_board.get_pico_pins())

# get the event loop
loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)

# instantiate pymata_express
try:
    board = telemetrix_rpi_pico_2w_serial_aio.TelemetrixRpiPico2WSerialAIO(loop=loop)
except (KeyboardInterrupt, RuntimeError):
    # loop.run_until_complete(board.reset_board())
    sys.exit()

try:
    # start the main function
    loop.run_until_complete(get_pin_report(board))
    loop.run_until_complete(board.reset_board())
except KeyboardInterrupt:
    loop.run_until_complete(board.shutdown())
    sys.exit(0)
except RuntimeError:
    sys.exit(0)



# SPDX-FileCopyrightText: 2020 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

# Connect to an "eval()" service over BLE UART.

from adafruit_ble import BLERadio
from adafruit_ble.advertising.standard import ProvideServicesAdvertisement
from adafruit_ble.services.nordic import UARTService

ble = BLERadio()

uart_connection = None

while True:
    if not uart_connection:
        print("Trying to connect...")
        for adv in ble.start_scan(ProvideServicesAdvertisement):
            if UARTService in adv.services:
                uart_connection = ble.connect(adv)
                print("Connected")
                break
        ble.stop_scan()

    if uart_connection and uart_connection.connected:
        uart_service = uart_connection[UARTService]
        while uart_connection.connected:
            s = input("Eval: ")
            uart_service.write(s.encode("utf-8"))
            uart_service.write(b'\n')
            print(uart_service.readline().decode("utf-8"))
2+2


from adafruit_ble import BLERadio
from adafruit_ble.advertising.standard import ProvideServicesAdvertisement
from adafruit_ble.services.nordic import UARTService
import time

ble = BLERadio()
uart_connection = None

UART_RX_CHAR_UUID = "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"

while True:
    if not uart_connection:
        print("Trying to connect...")
        for adv in ble.start_scan(ProvideServicesAdvertisement, timeout=5):
            if UARTService in adv.services:
                uart_connection = ble.connect(adv)
                print("Connected")
                break
        ble.stop_scan()

    if uart_connection and uart_connection.connected:
        # Access the underlying bleak client directly
        bleak_client = uart_connection._bleio_connection._bleak_client

        data = bytes([1, 62])
        print(f"Writing directly to RX characteristic: {list(data)}")

        # Write directly to the correct characteristic
        import asyncio

        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            bleak_client.write_gatt_char(UART_RX_CHAR_UUID, data)
        )

        print("Data written")
        time.sleep(2)
        uart_connection.disconnect()
        break


"""
 Copyright (c) 2021-2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import sys
import time
from telemetrix_rpi_pico_2w_serial import telemetrix_rpi_pico_2w_serial


"""
This program monitors a DHT device
"""
DHT_PIN = 6

# indices into callback data for valid data
REPORT_TYPE = 0
REPORT_SUBTYPE = 1 # 0 = good data. Non-zero = error value returned
PIN = 2
HUMIDITY = 3
TEMPERATURE = 4
TIME = 5
ERROR_TIME = 3


# A callback function to display the distance
def the_callback(data):
    """
    The callback function to display the current humidity and temperature
    :param data: [report_type = PrivateConstants.DHT_REPORT, pin  humidity,
                                temperature (Celsius), timestamp]

    """

    # pretty print the info
    # if data[0] is not zero than this is an error report
    if data[REPORT_SUBTYPE] != 0:
        date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[ERROR_TIME]))

        print(f'DHT Error Report:  '
              f'Pin: {data[PIN]} Error Value: {data[REPORT_SUBTYPE]} Time: {date}')
    # good data
    else:
        date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[TIME]))
        temperature_c = data[TEMPERATURE]
        temperature_f = temperature_c * 9/5 + 32
        print(f'DHT Data Report:'   
              f'Pin: {data[PIN]} Humidity: {data[HUMIDITY]} Temperature:  '
              f'{temperature_c}c  {temperature_f}f  Time: {date}')


# instantiate a pico
board = telemetrix_rpi_pico_2w_serial.TelemetrixRpiPico2wSerial()

# set a pin to DHT mode
board.set_pin_mode_dht(DHT_PIN, the_callback)

# wait forever
while True:
    try:
        time.sleep(.01)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)




"""
 Copyright (c) 2021-2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 """

import sys
import time

from telemetrix_rpi_pico_2w_serial import telemetrix_rpi_pico_2w_serial

"""
This is an example program for HC-SR04 type distance sensors.

"""


# some globals
TRIGGER_PIN = 4
ECHO_PIN = 5

# indices into callback data
TRIGGER = 1
DISTANCE_IN_CENTIMETERS = 2
TIME_STAMP = 3

MAX_TIME_TO_WAIT_FOR_REPORT = 3  # in seconds

# initialize to current time
last_report_received = time.time()


def the_callback(data):
    date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[TIME_STAMP]))

    global last_report_received
    last_report_received = time.time()

    print(f'{date}\t Trigger Pin::\t{data[TRIGGER]}\t Distance(cm):\t'
          f'{data[DISTANCE_IN_CENTIMETERS]}')


# Create a Telemetrix instance.
board = telemetrix_rpi_pico_2w_serial.TelemetrixRpiPico2wSerial()
try:
    # instantiate HC-SR04 devices
    board.set_pin_mode_sonar(TRIGGER_PIN, ECHO_PIN, the_callback)

    while True:
        try:
            # do nothing but sleep while the reports come in.
            time.sleep(1)
            current_time = time.time()
            if current_time - last_report_received > MAX_TIME_TO_WAIT_FOR_REPORT:
                print("No response from HC-SR04")
                board.shutdown()
                sys.exit(0)
        except KeyboardInterrupt:
            board.shutdown()
            sys.exit(0)

except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)



"""
 Copyright (c) 2021-2026 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import time
import sys
from telemetrix_rpi_pico_2w_ble import telemetrix_rpi_pico_2w_ble

"""
Demo using an 8 LED NeoPixel strip
"""


def neopixel_demo(my_board):
    """
    Run
    :param my_board: Pico board instance
    """

    # enable neopixel support on the Pico pin 0
    my_board.set_pin_mode_neopixel(pin_number=0)

    # set some values and the show them
    my_board.neo_pixel_set_value(5, 255, 0, 0)
    my_board.neo_pixel_set_value(1, 0, 64, 0)
    my_board.neo_pixel_set_value(7, 0, 0, 64)
    my_board.neopixel_show()

    time.sleep(1)

    # clear the NeoPixels
    my_board.neopixel_clear()

    time.sleep(1)

    # fill the NeoPixels
    my_board.neopixel_fill(50, 0, 120)

    time.sleep(1)

    # set pixel value and update immediately
    my_board.neo_pixel_set_value(3, 0, 65, 64, True)
    time.sleep(1)

    my_board.neopixel_clear()
    # pixel sequence
    for pixel in range(8):
        my_board.neo_pixel_set_value(pixel, 0, 0, 64, True)
        time.sleep(.1)
        my_board.neopixel_clear()
        time.sleep(.01)


board = telemetrix_rpi_pico_2w_ble.TelemetrixRpiPico2Ble()

try:
    neopixel_demo(board)

    board.shutdown()

except KeyboardInterrupt:
    board.neopixel_clear()

    board.shutdown()
    sys.exit(0)



"""
 Copyright (c) 2020-2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import asyncio
import sys
from telemetrix_rpi_pico_2w_ble_aio import telemetrix_rpi_pico_2w_ble_aio

"""
This example sets up and control an ADXL345 i2c accelerometer.
It will continuously print data the raw xyz data from the device.

SDA = pin 4   SCL = pin 5
"""


# the call back function to print the adxl345 data
async def the_callback(data):
    """

    :param data: [pin_type, Device address, device read register, x data pair, y data pair, z data pair]
    :return:
    """
    print(data)


async def adxl345(my_board):
    # setup adxl345
    # device address = 83
    await my_board.set_pin_mode_i2c(0)

    # set up power and control register
    await my_board.i2c_write(83, [45, 0], i2c_port=0)
    await asyncio.sleep(.1)
    await my_board.i2c_write(83, [45, 8], i2c_port=0)
    await asyncio.sleep(.1)

    # set up the data format register
    await my_board.i2c_write(83, [49, 8], i2c_port=0)
    await asyncio.sleep(.1)
    await my_board.i2c_write(83, [49, 3], i2c_port=0)
    await asyncio.sleep(.1)

    # read_count = 20
    while True:
        # read 6 bytes from the data register
        try:
            await my_board.i2c_read(83, 50, 6, callback=the_callback, i2c_port=0)
            await asyncio.sleep(.1)

        except (KeyboardInterrupt, RuntimeError):
            await my_board.shutdown()
            sys.exit(0)


# get the event loop
loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)

# instantiate telemetrix
try:
    board = telemetrix_rpi_pico_2w_ble_aio.TelemetrixRpiPico2wBleAio(ble_device_name=
                                                                    "Tmx4Pico2W",
                                                                    loop=loop)
except (KeyboardInterrupt, RuntimeError):
    # loop.run_until_complete(board.reset_board())
    sys.exit()

try:
    # start the main function
    loop.run_until_complete(adxl345(board))
except KeyboardInterrupt:
    loop.run_until_complete(board.shutdown())
    sys.exit(0)



"""
 Example demonstrating the synchronous BLE transport for Telemetrix
 on Raspberry Pi Pico 2W

 This example shows how to:
 - Connect to a Pico 2W over BLE
 - Send commands using the Telemetrix protocol
 - Receive and process data from the Pico
"""

import time
import sys
from telemetrix_rpi_pico_2w_ble import TelemetrixRpiPico2wBle


# Telemetrix command constants (from your protocol)
# You'll need to define these based on your actual protocol
class Commands:
    """Telemetrix command opcodes"""
    LOOP_COMMAND = 1
    SET_PIN_MODE = 2
    DIGITAL_WRITE = 3
    ANALOG_WRITE = 4
    # Add more as needed from your protocol


class TelemetrixBleExample:
    """
    Example class showing how to use the synchronous BLE transport
    with Telemetrix protocol
    """

    def __init__(self, device_name="Tmx4Pico2W"):
        """
        Initialize the example

        :param device_name: BLE device name to connect to
        """
        self.device_name = device_name
        self.board = None
        self.receive_buffer = bytearray()

    def data_receive_callback(self, sender, data):
        """
        Callback to handle incoming data from the Pico.
        This is where you'd parse Telemetrix protocol responses.

        :param sender: BLE characteristic that sent the data
        :param data: bytes received
        """
        print(f"Received {len(data)} bytes: {list(data)}")

        # Add to receive buffer
        self.receive_buffer.extend(data)

        # Process complete messages
        # This is a simple example - adjust based on your protocol
        while len(self.receive_buffer) >= 2:  # Assuming 2-byte messages
            # Extract message
            message = bytes(self.receive_buffer[:2])
            self.receive_buffer = self.receive_buffer[2:]

            # Parse message based on Telemetrix protocol
            self.parse_telemetrix_message(message)

    def parse_telemetrix_message(self, message):
        """
        Parse a Telemetrix protocol message

        :param message: bytes to parse
        """
        if len(message) < 1:
            return

        command = message[0]

        # Example parsing - adjust based on your actual protocol
        if command == Commands.LOOP_COMMAND:
            print("Received loop command response")
        else:
            print(f"Received command {command} with data: {list(message[1:])}")

    def connect(self):
        """Connect to the BLE device"""
        print(f"Connecting to {self.device_name}...")

        self.board = TelemetrixRpiPico2wBle(
            self.device_name,
            receive_callback=self.data_receive_callback
        )

        if self.board.connect(timeout=15):
            print("Successfully connected!")
            return True
        else:
            print("Failed to connect")
            return False

    def send_command(self, command_bytes):
        """
        Send a Telemetrix command to the Pico

        :param command_bytes: list of bytes or bytes object to send
        """
        if isinstance(command_bytes, list):
            print(f"Sending command: {command_bytes}")
        else:
            print(f"Sending command: {list(command_bytes)}")

        return self.board.write(command_bytes)

    def disconnect(self):
        """Disconnect from the BLE device"""
        if self.board:
            print("Disconnecting...")
            self.board.disconnect()
            print("Disconnected")

    def run_example(self):
        """Run a simple example sequence"""
        try:
            # Connect
            if not self.connect():
                return

            # Give connection time to stabilize
            time.sleep(1)

            # Example 1: Send a simple 2-byte command
            print("\n=== Example 1: Simple Command ===")
            self.send_command([1, 62])
            time.sleep(1)

            # Example 2: Send loop command (keep-alive)
            print("\n=== Example 2: Loop Command ===")
            self.send_command([Commands.LOOP_COMMAND, 0])
            time.sleep(1)

            # Example 3: Set pin mode (example - adjust for your protocol)
            print("\n=== Example 3: Set Pin Mode ===")
            # Command format might be: [SET_PIN_MODE, pin_number, mode]
            pin = 25  # LED pin on Pico
            mode = 1  # OUTPUT mode
            self.send_command([Commands.SET_PIN_MODE, pin, mode])
            time.sleep(1)

            # Example 4: Digital write
            print("\n=== Example 4: Digital Write ===")
            # Command format might be: [DIGITAL_WRITE, pin_number, value]
            self.send_command([Commands.DIGITAL_WRITE, pin, 1])  # Turn on
            time.sleep(1)

            self.send_command([Commands.DIGITAL_WRITE, pin, 0])  # Turn off
            time.sleep(1)

            # Example 5: Multiple rapid commands
            print("\n=== Example 5: Blink LED ===")
            for i in range(5):
                print(f"Blink {i + 1}/5")
                self.send_command([Commands.DIGITAL_WRITE, pin, 1])
                time.sleep(0.3)
                self.send_command([Commands.DIGITAL_WRITE, pin, 0])
                time.sleep(0.3)

            print("\n=== Example complete ===")

        except KeyboardInterrupt:
            print("\nInterrupted by user")
        except Exception as e:
            print(f"Error: {e}")
            import traceback
            traceback.print_exc()
        finally:
            # Always disconnect
            self.disconnect()


def main():
    """Main entry point"""
    # You can specify a different device name if needed
    device_name = "Tmx4Pico2W"

    if len(sys.argv) > 1:
        device_name = sys.argv[1]

    print("=" * 60)
    print("Telemetrix BLE Example")
    print("=" * 60)
    print(f"Target device: {device_name}")
    print("Press Ctrl+C to exit")
    print("=" * 60)

    example = TelemetrixBleExample(device_name)
    example.run_example()


if __name__ == "__main__":
    main()


"""
 Copyright (c) 2021-2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import asyncio
import sys
from telemetrix_rpi_pico_2w_serial_aio import telemetrix_rpi_pico_2w_serial_aio

"""
Demo using an 8 LED NeoPixel strip
"""


"""
Demo using an 8 LED NeoPixel strip
"""


async def neopixel_demo(my_board):
    """
    Run
    :param my_board: Pico board instance
    """

    # enable neopixel support on the Pico pin 0
    await my_board.set_pin_mode_neopixel(pin_number=0)

    # set some values and the show them
    await my_board.neo_pixel_set_value(5, 255, 0, 0)
    await my_board.neo_pixel_set_value(1, 0, 64, 0)
    await my_board.neo_pixel_set_value(7, 0, 0, 64)
    await my_board.neopixel_show()

    await asyncio.sleep(1)

    # clear the NeoPixels
    await my_board.neopixel_clear()

    await asyncio.sleep(1)

    # fill the NeoPixels
    await my_board.neopixel_fill(50, 0, 120)

    await asyncio.sleep(1)

    # set pixel value and update immediately
    await my_board.neo_pixel_set_value(3, 0, 65, 64, True)
    await asyncio.sleep(1)

    await my_board.neopixel_clear()
    # pixel sequence
    for pixel in range(8):
        await my_board.neo_pixel_set_value(pixel, 0, 0, 64, True)
        await asyncio.sleep(.1)
        await my_board.neopixel_clear()
        await asyncio.sleep(.01)


# get the event loop
loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)

board = telemetrix_rpi_pico_2w_serial_aio.TelemetrixRpiPico2WSerialAIO(loop=loop)

# start the main function
try:
    loop.run_until_complete(neopixel_demo(board))
    loop.run_until_complete(board.neopixel_clear())
    loop.run_until_complete(board.shutdown())
except KeyboardInterrupt:
    loop.run_until_complete(board.neopixel_clear())
    loop.run_until_complete(board.shutdown())
    sys.exit(0)
except RuntimeError:
    sys.exit(0)




"""
 Copyright (c) 2025 Alan Yorinks All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 Version 3 as published by the Free Software Foundation; either
 or (at your option) any later version.
 This library is distributed in the hope that it will be useful,f
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
 along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

import sys
import time
from telemetrix_rpi_pico_2w_serial import telemetrix_rpi_pico_2w_serial


#
"""
Monitor the Pico internal temperature sensor and return the temperature
in celsius in the callback.
"""

# Callback data indices
CB_REPORT_TYPE = 0
CB_TEMP = 1
CB_TIME = 2
board = telemetrix_rpi_pico_2w_serial.TelemetrixRpiPico2wSerial()


def the_callback(data):
    """
    A callback function to report data changes.
    This will print the pin number, its reported value and
    the date and time when the differential is exceeded
    :param data: [report_type, temperature, timestamp]
    """
    print(f'raw data = {data}')
    date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(data[CB_TIME]))

    print(f'CPU Temperature: {data[CB_TEMP]} Date: {date}')


def get_cpu_temp():
    """
     This function will request cpu temperature reports
     """
    # board = telemetrix_rpi_pico_2w_serial.TelemetrixRpiPico2wSerial()

    # set the pin mode
    board.get_cpu_temperature(threshold=.01, polling_interval=3000,
                              callback=the_callback)
    time.sleep(.4)

    print('Enter Control-C to quit.')
    try:

        while True:
            time.sleep(.1)
    except KeyboardInterrupt:
        board.shutdown()
        sys.exit(0)


try:
    get_cpu_temp()
except KeyboardInterrupt:
    board.shutdown()
    sys.exit(0)
